/**
 *    @file  libhttp.h
 *   @brief  HTTP parsing and building library
 *
 *  <+DETAILED+>
 *
 *  @author  François Cerbelle (Fanfan), francois@cerbelle.net
 *
 *  @internal
 *       Created:  19/11/2024
 *      Revision:  none
 * Last modified:  2024-11-22 08:26
 *      Compiler:  gcc
 *  Organization:  Cerbelle.net
 *     Copyright:  Copyright (c) 2024, François Cerbelle
 *
 *  This source code is released for free distribution under the terms of the
 *  GNU General Public License as published by the Free Software Foundation.
 */

#ifndef __LIBHTTP_H__
#define __LIBHTTP_H__

typedef enum HTTPVersion_e {
    HTTPVERSION_HTTP09,
    HTTPVERSION_HTTP10,
    HTTPVERSION_HTTP11,
    HTTPVERSION_HTTP11b,
    HTTPVERSION_HTTP2,
    HTTPVERSION_HTTP3,
    HTTPVERSION_INVALID
} HTTPVersion_t;

typedef enum HTTPMethod_e {
    HTTPMETHOD_GET,
    HTTPMETHOD_HEAD,
    HTTPMETHOD_POST,
    HTTPMETHOD_PUT,
    HTTPMETHOD_DELETE,
    HTTPMETHOD_CONNECT,
    HTTPMETHOD_OPTIONS,
    HTTPMETHOD_TRACE,
    HTTPMETHOD_PATCH,
    HTTPMETHOD_INVALID
} HTTPMethod_t;

typedef struct HTTPHeader_s  HTTPHeader_t;

HTTPHeader_t* HTTPHeader_new(const char* name, const char* value);
void          HTTPHeader_del(HTTPHeader_t* header);
HTTPHeader_t* HTTPHeader_basicauth(const char* login, const char* pass);
char*         HTTPHeader_getname(HTTPHeader_t* header);
char*         HTTPHeader_getvalue(HTTPHeader_t* header);

typedef struct HTTP_s HTTP_t;

HTTP_t*       HTTP_new();
void          HTTP_del(HTTP_t* http);
char*         HTTP_setbody(HTTP_t* http, const char* body);
char*         HTTP_getbody(HTTP_t* http);
HTTP_t*       HTTP_addheader(HTTP_t* http, HTTPHeader_t* header);
HTTPHeader_t* HTTP_firstheader(const HTTP_t* http);
HTTPHeader_t* HTTP_nextheader(const HTTPHeader_t* header);
HTTPHeader_t* HTTP_findheader(const HTTPHeader_t* start, const char* name, const char* value);
HTTP_t*       HTTP_delheader(HTTP_t* http, HTTPHeader_t* header);

char*         HTTP_buildrequest(const HTTPMethod_t method, const char* uri, const HTTPVersion_t version);
char*         HTTP_buildheaders(const HTTP_t* http);
/*
void          HTTP_parserequest(const char* request, HTTP_t** http, HTTP_Method_t method, char* uri, HTTPVersion_t version);
 * typedef enum   HTTPStatus_e  HTTPStatus_t;
 *
 * char*         HTTP_buildreply(const HTTP_t* http, const HTTPVersion_t, const HTTPStatus_t);
 * void          HTTP_parsereply(const char* reply, HTTP_t** http, HTTPVersion_t*, HTTPStatus_t*);
 */

#endif /* __LIBHTTP_H__ */
